@@ -1,5 +1,5 @@
 use crate::{
-    ast::{FromInputValue, InputValue},
+    ast::{Document, FromInputValue, InputValue},
     executor::Registry,
     parser::parse_document_source,
     schema::{
@@ -812,20 +812,13 @@ where
     }
 }
 
-pub fn validate<'a, Q, M, Sub, V, F, S>(
-    r: Q,
-    m: M,
-    s: Sub,
-    q: &'a str,
-    factory: F,
-) -> Vec<RuleError>
+pub fn validate<'a, Q, M, Sub, F, S>(r: Q, m: M, s: Sub, q: &'a str, visit_fn: F) -> Vec<RuleError>
 where
     S: ScalarValue + 'a,
     Q: GraphQLType<S, TypeInfo = ()>,
     M: GraphQLType<S, TypeInfo = ()>,
     Sub: GraphQLType<S, TypeInfo = ()>,
-    V: Visitor<'a, S> + 'a,
-    F: Fn() -> V,
+    F: FnOnce(&mut ValidatorContext<'a, S>, &'a Document<S>),
 {
     let mut root = RootNode::new_with_scalar_value(r, m, s);
 
@@ -864,10 +857,7 @@ where
         parse_document_source(q, &root.schema).expect(&format!("Parse error on input {:#?}", q));
     let mut ctx = ValidatorContext::new(unsafe { ::std::mem::transmute(&root.schema) }, &doc);
 
-    let mut mv = MultiVisitorNil.with(factory());
-    visit(&mut mv, &mut ctx, unsafe {
-        ::std::mem::transmute(doc.as_slice())
-    });
+    visit_fn(&mut ctx, unsafe { ::std::mem::transmute(doc.as_slice()) });
 
     ctx.into_errors()
 }
@@ -881,6 +871,14 @@ where
     expect_passes_rule_with_schema(QueryRoot, MutationRoot, SubscriptionRoot, factory, q);
 }
 
+pub fn expect_passes_fn<'a, F, S>(visit_fn: F, q: &'a str)
+where
+    S: ScalarValue + 'a,
+    F: FnOnce(&mut ValidatorContext<'a, S>, &'a Document<S>),
+{
+    expect_passes_fn_with_schema(QueryRoot, MutationRoot, SubscriptionRoot, visit_fn, q);
+}
+
 pub fn expect_passes_rule_with_schema<'a, Q, M, Sub, V, F, S>(
     r: Q,
     m: M,
@@ -893,9 +891,12 @@ pub fn expect_passes_rule_with_schema<'a, Q, M, Sub, V, F, S>(
     M: GraphQLType<S, TypeInfo = ()>,
     Sub: GraphQLType<S, TypeInfo = ()>,
     V: Visitor<'a, S> + 'a,
-    F: Fn() -> V,
+    F: FnOnce() -> V,
 {
-    let errs = validate(r, m, s, q, factory);
+    let errs = validate(r, m, s, q, move |ctx, doc| {
+        let mut mv = MultiVisitorNil.with(factory());
+        visit(&mut mv, ctx, unsafe { ::std::mem::transmute(doc) });
+    });
 
     if !errs.is_empty() {
         print_errors(&errs);
@@ -903,6 +904,27 @@ pub fn expect_passes_rule_with_schema<'a, Q, M, Sub, V, F, S>(
     }
 }
 
+pub fn expect_passes_fn_with_schema<'a, Q, M, Sub, F, S>(
+    r: Q,
+    m: M,
+    s: Sub,
+    visit_fn: F,
+    q: &'a str,
+) where
+    S: ScalarValue + 'a,
+    Q: GraphQLType<S, TypeInfo = ()>,
+    M: GraphQLType<S, TypeInfo = ()>,
+    Sub: GraphQLType<S, TypeInfo = ()>,
+    F: FnOnce(&mut ValidatorContext<'a, S>, &'a Document<S>),
+{
+    let errs = validate(r, m, s, q, visit_fn);
+
+    if !errs.is_empty() {
+        print_errors(&errs);
+        panic!("Expected `visit_fn` to pass, but errors found");
+    }
+}
+
 pub fn expect_fails_rule<'a, V, F, S>(factory: F, q: &'a str, expected_errors: &[RuleError])
 where
     S: ScalarValue + 'a,
@@ -912,6 +934,14 @@ where
     expect_fails_rule_with_schema(QueryRoot, MutationRoot, factory, q, expected_errors);
 }
 
+pub fn expect_fails_fn<'a, F, S>(visit_fn: F, q: &'a str, expected_errors: &[RuleError])
+where
+    S: ScalarValue + 'a,
+    F: FnOnce(&mut ValidatorContext<'a, S>, &'a Document<S>),
+{
+    expect_fails_fn_with_schema(QueryRoot, MutationRoot, visit_fn, q, expected_errors);
+}
+
 pub fn expect_fails_rule_with_schema<'a, Q, M, V, F, S>(
     r: Q,
     m: M,
@@ -923,9 +953,18 @@ pub fn expect_fails_rule_with_schema<'a, Q, M, V, F, S>(
     Q: GraphQLType<S, TypeInfo = ()>,
     M: GraphQLType<S, TypeInfo = ()>,
     V: Visitor<'a, S> + 'a,
-    F: Fn() -> V,
+    F: FnOnce() -> V,
 {
-    let errs = validate(r, m, crate::EmptySubscription::<S>::new(), q, factory);
+    let errs = validate(
+        r,
+        m,
+        crate::EmptySubscription::<S>::new(),
+        q,
+        move |ctx, doc| {
+            let mut mv = MultiVisitorNil.with(factory());
+            visit(&mut mv, ctx, unsafe { ::std::mem::transmute(doc) });
+        },
+    );
 
     if errs.is_empty() {
         panic!("Expected rule to fail, but no errors were found");
@@ -940,6 +979,33 @@ pub fn expect_fails_rule_with_schema<'a, Q, M, V, F, S>(
     }
 }
 
+pub fn expect_fails_fn_with_schema<'a, Q, M, F, S>(
+    r: Q,
+    m: M,
+    visit_fn: F,
+    q: &'a str,
+    expected_errors: &[RuleError],
+) where
+    S: ScalarValue + 'a,
+    Q: GraphQLType<S, TypeInfo = ()>,
+    M: GraphQLType<S, TypeInfo = ()>,
+    F: FnOnce(&mut ValidatorContext<'a, S>, &'a Document<S>),
+{
+    let errs = validate(r, m, crate::EmptySubscription::<S>::new(), q, visit_fn);
+
+    if errs.is_empty() {
+        panic!("Expected `visit_fn`` to fail, but no errors were found");
+    } else if errs != expected_errors {
+        println!("==> Expected errors:");
+        print_errors(expected_errors);
+
+        println!("\n==> Actual errors:");
+        print_errors(&errs);
+
+        panic!("Unexpected set of errors found");
+    }
+}
+
 fn print_errors(errs: &[RuleError]) {
     for err in errs {
         for p in err.locations() {
